/*
 * $Id: b64.c,v 1.2 2002/10/15 08:41:09 skyper Exp $
 */
#include <stdarg.h>
#include <unistd.h>
#include <string.h>


/*
 * make it my-base64 encoded.
 * This is not really base64 encoding. Why should we use
 * a converting table if we can make it faster with +32 if (and only
 * if) chars from 32..32+64 are allowed by the IRC protocol?
 * The final string is \0 terminated.
 * 127 is threaded as THCFRAGMENT packet indicator. see irc.h
 */
size_t
b64_enc(unsigned char *buf, size_t blen, const unsigned char *src, int slen)
{
	unsigned char c;
	const unsigned char *end = src + slen;
	unsigned char *start = buf;
	
	while ((src < end) && (buf - start + 4 < blen))
	{
		*buf++ = (*src >> 2) + 32;

		c = (*src++ & 0x03)<<4;
		if (src >= end)
		{
			*buf++ = c + 32;
			break;
		}
		*buf++ =  (c | ((*src & ~0x0f)>>4)) + 32;

		c = (*src++ & 0x0f)<<2;
		if (src >= end)
		{
			*buf++ = c + 32;
			break;
		}
		*buf++ = (c | (*src>>6)) + 32;

		*buf++ = (*src++ & 0x3f) + 32;
	}
	*buf = '\0';

	return buf - start;
}

/*
 * the final string is not 0 terminated
 */
size_t
b64_dec(unsigned char *buf, size_t len, const unsigned char *src, int slen)
{
	const unsigned char *end = src + slen;
	unsigned char c;
	unsigned char *start = buf;

	while ((src < end) && (buf - start + 3 < len))
	{
		c = ((*src++ - 32) << 2);
		if (src >= end)
		{
			*buf++ = c;
			break;
		}
		c = c | (((*src - 32)>>4) & 0x03);
		*buf++ = c;

		c = (*src++ - 32) << 4;
		if (src >= end)
		{
			*buf = c;
			return buf - start;
		}
		*buf++ = c | ((*src - 32) >> 2);

		*buf = (*src++ - 32) << 6;
		if (src >= end)
			break;

		*buf++ |= (*src++ - 32);
	}

	return buf - start;
}

