/*
 * $Id: system.c,v 1.2 2002/10/15 09:24:28 skyper Exp $
 */

#include "common.h"
#include <sys/stat.h>
#include <sys/types.h>
#include <string.h>
#include <stdio.h>
#include "system.h"

/*
 * Create directory hirachy recursively
 */
int
mkpdir(const char *pathname, mode_t mode)
{
	char buf[1024];
	char *ptr = buf;
	char *next;
	struct stat sbuf;

	strlcpy(buf, pathname, sizeof buf);


	while (ptr)
	{
		while (*ptr == '/')
			ptr++;

		next = strchr(ptr, '/');
		if (next)
			*next = '\0';
		if (stat(buf, &sbuf) == 0)
		{
			if (!S_ISDIR(sbuf.st_mode))
				return -1;
		} else {
			if (mkdir(buf, 0700) != 0)
				return -1;
		}

		if (next)
			*next++ = '/';
		ptr = next;
	}

	return 0;
}

#ifndef HAVE_STRLCPY
/*
 * bsd'sh strlcpy().
 * The strlcpy() function copies up to size-1 characters from the
 * NUL-terminated string src to dst, NUL-terminating the result.
 * Return: total length of the string tried to create.
 */
size_t
strlcpy(char *dst, const char *src, size_t size)
{
	size_t len = strlen(src);
	size_t ret = len;

	if (size <= 0)
		return 0;
	if (len >= size)
		len = size - 1;
	memcpy(dst, src, len);
	dst[len] = 0;

	return ret;
}
#endif

#ifndef HAVE_STRLCAT
/*
 * bsd'sh strlcat().
 * The strlcat() function appends the NUL-terminated string src to the end of
 * dst. It will append at most size - strlen(dst) - 1 bytes, NUL-terminating
 * the result.
 */
size_t
strlcat(char *dst, const char *src, size_t size)
{
	size_t len1 = strlen(dst);
	size_t len2 = strlen(src);
	size_t ret = len1 + len2;

	if (len1 + len2 >= size)
		len2 = size - (len1 + 1);

	if (len2 > 0) {
		memcpy(dst + len1, src, len2);
		dst[len1 + len2] = 0;
	}
	
	return ret;
}
#endif

#ifndef HAVE_STRLNCAT
/*
 * see strlcat(), but never cat more then n characters.
 */
size_t
strlncat(char *dst, const char *src, size_t size, size_t n)
{
	size_t len1 = strlen(dst);
	size_t len2 = strlen(src);
	size_t ret = len1 + len2;

	if (len2 > n)
		len2 = n;

	if (len1 + len2 >= size)
		len2 = size - (len1 + 1);

	if (len2 > 0) {
		memcpy(dst + len1, src, len2);
		dst[len1 + len2] = 0;
	}

	return ret;
}
#endif


