/*
 * $Id: user.c,v 1.2 2002/10/15 08:41:09 skyper Exp $
 */

#include "common.h"
#include <string.h>
#include <openssl/dh.h>
#include "user.h"
#include "system.h"

#define THCIRC_MAX_USERS         (256)
#define THC_HASHSZ               (32)

static USER *users[THC_HASHSZ];     /* HASH for user/nicks        */
static int n_users;   /* Number of users in storage */
static char *state_string[] = {"EMPTY", "NO RESPONSE", "NOT AUTH", "NOT AUTH", "EST"};

static int USER_hash(const char *nick);

char *
USER_state(USER *u)
{
	if (u->state >= sizeof state_string / sizeof *state_string)
		return "OVERFLOW";

	return state_string[(int)u->state];
}

/*
 * Return number of Users in hash table.
 */
int
USER_n(void)
{
	return n_users;
}

/*
 * Return next user in Hash table (no sorting).
 */
USER *
USER_next(USER *u)
{
	int i = 0;

	if (u)
	{
		if (u->next)
			return u->next;
		i = USER_hash(u->nick) + 1;
	}

	for (; i < THC_HASHSZ; i++)
		if (users[i] != NULL)
			return users[i];

	return NULL;
}

static int
USER_hash(const char *nick)
{
	int c = 0;
	int n;

	n = strlen(nick);

	while (*nick)
		c += *nick++;

	return c % THC_HASHSZ;
}


USER *
USER_lookup(const char *nick)
{
	USER *user;

	user = users[USER_hash(nick)];

	while (user)
	{
		if (!strcmp(nick, user->nick))
			break;
		user = user->next;
	}

	return user;
}

USER *
USER_add(const char *nick)
{
	USER *user;
	int c;

	if (n_users >= THCIRC_MAX_USERS)
		return NULL;

	if ( (user = calloc(1, sizeof *user)) == NULL)
		return NULL;

	n_users++;
	strlcpy(user->nick, nick, sizeof user->nick);

	c = USER_hash(nick);
	if (users[c] == NULL)
	{
		users[c] = user;
		return user;
	}

	user->next = users[c]->next;
	users[c]->next = user;

	return user;
}

void
USER_del(char *nick)
{
	USER *u;
	int c = USER_hash(nick);

	if (users[c] == NULL)
		return;

	if (!strcmp(users[c]->nick, nick))
	{
		u = users[c];
		users[c] = users[c]->next;
		goto found;
	}

	u = users[c];

	while (u->next)
	{
		if (!strcmp(u->next->nick, nick))
		{
			u->next = u->next->next;
			u = u->next;
			goto found;
		}
	}

	return;

found:
	if (n_users > 0)
		n_users--;
	USER_free_usm(u);
	free(u);
}

void
USER_free_usm(USER *u)
{
	if (!u->usm)
		return;

	if (u->usm->dh)
		DH_free(u->usm->dh);
	/* Destroy key memory */
	memset(u->usm, 0, sizeof u->usm);
	free(u->usm);
	u->usm = NULL;
}

/*
 * Free all old key material
 * and reset all values to 0
 */
int
USER_init(USER *u)
{
	USER_free_usm(u);
	u->usm = calloc(1, sizeof(struct user_sm));
	u->dec.hmac_seq = 0;
	u->enc.hmac_seq = 0;
	u->state = 0;

	/* FIXME: free enc and dec key material */

	return 0;
}

